@@ -321,7 +321,7 @@ static int vorbis_header(AVFormatContext *s, int idx)
     if (priv->packet[pkt_type >> 1])
         return AVERROR_INVALIDDATA;
     if (pkt_type > 1 && !priv->packet[0] || pkt_type > 3 && !priv->packet[1])
-        return AVERROR_INVALIDDATA;
+        return priv->vp ? 0 : AVERROR_INVALIDDATA;
 
     priv->len[pkt_type >> 1]    = os->psize;
     priv->packet[pkt_type >> 1] = av_mallocz(os->psize);
@@ -387,12 +387,7 @@ static int vorbis_header(AVFormatContext *s, int idx)
             }
         }
     } else {
-        int ret;
-
-        if (priv->vp)
-             return AVERROR_INVALIDDATA;
-
-        ret = fixup_vorbis_headers(s, priv, &st->codecpar->extradata);
+        int ret = fixup_vorbis_headers(s, priv, &st->codecpar->extradata);
         if (ret < 0) {
             st->codecpar->extradata_size = 0;
             return ret;
